#! /bin/bash

function set_computer_mode()
{
    echo "### switching mode: computer" >> ${LOG}
    echo "### cmd: xrandr --output ${M_MONITOR} --auto --output ${S_MONITOR} --off" >> ${LOG}
    xrandr --output ${M_MONITOR} --auto --output ${S_MONITOR} --off
}

function set_duplicate_mode()
{
    echo "### switching mode: duplicate" >> ${LOG}
    echo "xrandr --output ${M_MONITOR} --mode 1024x768 --primary --auto --output ${S_MONITOR} --mode 1024x768 --same-as ${M_MONITOR}" >> ${LOG}
    xrandr --output ${M_MONITOR} --mode 1024x768 --primary --auto --output ${S_MONITOR} --mode 1024x768 --same-as ${M_MONITOR}
}

function set_extension_mode()
{
    echo "### switching mode: extension " >> ${LOG}
    echo "### cmd:xrandr --output ${M_MONITOR} --auto --output ${S_MONITOR} --right-of ${M_MONITOR}" >> ${LOG}
    xrandr --output ${M_MONITOR}  --auto  --output ${S_MONITOR} --right-of ${M_MONITOR} --auto
}

function set_projector_mode()
{
    echo "### switching mode: projector" >> ${LOG}
    echo "### cmd:xrandr --output ${S_MONITOR} --auto --output ${M_MONITOR} --off" >> ${LOG}
    xrandr --output ${S_MONITOR} --auto --output ${M_MONITOR} --off
}

function get_current_mode()
{
    if [ "${N_NUM}" = "1" ];then 
        echo "### CAPTURE MODE: computer" >> ${LOG}
        touch ${C_FILE}
    elif [ "${N_NUM}" = "2" ]; then
        echo "### screen current: ${SCREEN_CUR} #M_MAX_RES: ${M_MAX_RES} #S_MAX_RES: ${S_MAX_RES}" >> ${LOG}
        temp_screen_cur_l=`echo ${SCREEN_CUR} | awk -Fx '{print $1}'`
        temp_m_max_res_l=`echo ${M_MAX_RES} | awk -Fx '{print $1}'`
        temp_s_max_res_l=`echo ${S_MAX_RES} | awk -Fx '{print $1}'`
        sum_m_s=`expr ${temp_m_max_res_l} + ${temp_s_max_res_l}`
        echo "### temp_screen_cur_l: ${temp_screen_cur_l} #temp_m_run_res_l: ${temp_m_run_res_l} #temp_s_run_res_l: ${temp_s_run_res_l}" >> ${LOG}

        num="$(xrandr | grep "*" | wc -l)" 
        if [ "${num}" = "1"  ];then
            echo "#####DEBUG: ${temp_m_max_res_l} # ${temp_screen_cur_l}" >> ${LOG}
            if [ "U"${temp_m_max_res_l} = "U"${temp_screen_cur_l} ];then
                echo "### CAPTURE MODE: computer" >> ${LOG}
                if [ ${MODE_SEC} -eq 1 ]; then 
                    echo "### MODE SEC = 1" >> ${LOG}
                    touch ${P_FILE}  
                else 
                    echo "### MODE SEC = 0" >> ${LOG}
                    touch ${C_FILE}
                fi
            elif [ "U"${temp_s_max_res_l} = "U"${temp_screen_cur_l} ];then
                echo "### CAPTURE MODE: projector" >> ${LOG}
                touch ${P_FILE}
            else
                echo "### CAPTURE MODE: unknown1" >> ${LOG}
            fi
        elif [ "${num}" = "2"  ];then
            #if [ ${temp_m_max_res_l} -eq ${temp_screen_cur_l} ];then
            echo "### sum_m_s: ${sum_m_s} # temp_screen_cur_l: ${temp_screen_cur_l}" >> ${LOG}
            if [ ${sum_m_s}"n" = ${temp_screen_cur_l}"n" ]; then
                echo "### CAPTURE MODE: extension" >> ${LOG}
                touch ${E_FILE}
            else
                echo "### CAPTURE MODE: duplicate" >> ${LOG}
                touch ${D_FILE}
            fi
        else
            echo "### CAPTURE MODE: unknown2" >> ${LOG}
        fi
    else
        echo "### CAPTURE MODE: unknown3" >> ${LOG}
    fi
}

function check_and_set_mode()
{
    mode=$1
    if [ "${mode}" = "COMPUTER" ]
    then
        set_computer_mode
    elif [ "${mode}" = "DUPLICATE" ]
    then
        set_duplicate_mode
    elif [ "${mode}" = "EXTENSION" ]
    then
        set_extension_mode
    elif [ "${mode}" = "PROJECTOR" ]
    then
        set_projector_mode
    else
        ### capture current mode. 
        get_current_mode 
    fi
}

function init_data()
{
    xrandr > ${INFO_DISPLAY}    
    num_m=0; num_s=0; mode=1
    while read line
    do
        # primary monitor max resolution
        if [ ${num_m} -eq 1 ]; then
            M_MAX_RES=`echo ${line} | awk '{print $1}'`
            echo "### PRIMARY MAX RES: ${M_MAX_RES}" >> ${LOG}
            num_m=0
        fi
        echo $line | grep "\<connected\>" | grep "eDP\|DP5\|LVDS1\|LVDS" >/dev/null; res_m=$?
        if [ ${res_m} -eq 0 ]; then num_m=1; fi

        # second monitor max resolution
        if [ ${num_s} -eq 1 ]; then
            S_MAX_RES=`echo ${line} | awk '{print $1}'`
            echo "### PRIMARY MAX RES: ${S_MAX_RES}" >> ${LOG}
            echo $line | grep "*" >/dev/null; mode_s=$?
            if [ ${mode_s} -eq 0 ]; then MODE_SEC=1; echo "### MODE_SEC = ${MODE_SEC}" >> ${LOG}; fi
            num_s=0
        fi
        echo $line | grep "\<connected\>" | grep "CRT1\|VGA-0\|VGA1" >/dev/null; res_s=$?
        if [ ${res_s} -eq 0 ]; then num_s=1; fi
    done < ${INFO_DISPLAY} 
   
    SCREEN_CUR="$(xrandr | grep Screen | awk -F, '{print $2}' | sed -r 's/ //g'| sed -r 's/current//g')"
    M_MONITOR="$(xrandr | grep -m 1 "\<connected\>" | cut -d ' ' -f1)"
    S_MONITOR="$(xrandr | grep "\<connected\>" | sed -n '2 p' | awk '{print $1}')"
    echo "### SCREEN_CUR:${SCREEN_CUR} #MNums:${N_NUM} #M_MONITOR: ${M_MONITOR} #S_MONITOR: ${S_MONITOR} #M_MAX_RES: ${M_MAX_RES} #S_MAX_RES: ${S_MAX_RES}" >> ${LOG}
}

function clear_temp_file_and_log()
{
    if [ -e ${LOG} ];then rm ${LOG}; touch ${LOG}; fi
    if [ -e ${C_FILE} ];then rm ${C_FILE}; fi
    if [ -e ${D_FILE} ];then rm ${D_FILE}; fi
    if [ -e ${E_FILE} ];then rm ${E_FILE}; fi
    if [ -e ${P_FILE} ];then rm ${P_FILE}; fi
}

### MAIN ### 
MODE="$1"; N_NUM="1"; M_MONITOR=""; S_MONITOR=""; M_MAX_RES=""; S_MAX_RES="";SCREEN_CUR=""
GXRANDR_TEMP_DIR=${HOME}/.local/share/gxrandr/
if [ ! -e ${GXRANDR_TEMP_DIR} ];then mkdir -p ${GXRANDR_TEMP_DIR}; fi
INFO_DISPLAY="${GXRANDR_TEMP_DIR}/xrandr.info"
#LOG="/dev/stdout"
LOG="${GXRANDR_TEMP_DIR}/xrandr.log"; touch ${LOG}
C_FILE="${GXRANDR_TEMP_DIR}/computer.mode"
D_FILE="${GXRANDR_TEMP_DIR}/duplicate.mode"
E_FILE="${GXRANDR_TEMP_DIR}/extension.mode"
P_FILE="${GXRANDR_TEMP_DIR}/projector.mode"
MODE_SEC=0

# clear all cache file.
clear_temp_file_and_log

N_NUM="$(xrandr | grep "\<connected\>" | wc -l)" 
if [ "${N_NUM}" = "2" ];then
    ### init global data
    init_data

    ### checking and set switching mode.
    check_and_set_mode ${MODE}

elif [ "${N_NUM}" = "1" ];then
    echo "### CAPTURE MODE: computer" >> ${LOG}
    touch ${C_FILE}
else
    echo "### ERROR, Please checking monitor number." >> ${LOG}
    exit -1
fi
